@@ -121,6 +121,7 @@ static const char *_lvname_has_reserved_string(const char *lvname)
 	static const char _strings[][12] = {
 		"_cdata",
 		"_cmeta",
+		"_corig",
 		"_mimage",
 		"_mlog",
 		"_pmspare",
@@ -186,7 +187,9 @@ char *build_dm_uuid(struct dm_pool *mem, const struct logical_volume *lv,
 		 * Should also make internal detection simpler.
 		 */
 		/* Suffixes used here MUST match lib/activate/dev_manager.c */
-		layer = lv_is_cache_pool_data(lv) ? "cdata" :
+		layer = lv_is_cache_origin(lv) ? "real" :
+			(lv_is_cache(lv) && lv_is_pending_delete(lv)) ? "real" :
+			lv_is_cache_pool_data(lv) ? "cdata" :
 			lv_is_cache_pool_metadata(lv) ? "cmeta" :
 			// FIXME: dm-tree needs fixes for mirrors/raids
 			//lv_is_mirror_image(lv) ? "mimage" :
