@@ -9,9 +9,9 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-. lib/test
+. lib/inittest
 
-aux target_at_least dm-raid 1 3 2 || skip
+aux have_raid 1 3 2 || skip
 
 aux prepare_vg 4
 
