@@ -20,11 +20,11 @@ aux have_raid 1 3 0 || skip
 
 aux prepare_vg 5
 
-lvcreate -s -l 20%FREE -n $lv1 $vg --virtualsize 256T
-lvcreate -s -l 20%FREE -n $lv2 $vg --virtualsize 256T
-lvcreate -s -l 20%FREE -n $lv3 $vg --virtualsize 256T
-lvcreate -s -l 20%FREE -n $lv4 $vg --virtualsize 256T
-lvcreate -s -l 20%FREE -n $lv5 $vg --virtualsize 256T
+lvcreate --type snapshot -s -l 20%FREE -n $lv1 $vg --virtualsize 256T
+lvcreate --type snapshot -s -l 20%FREE -n $lv2 $vg --virtualsize 256T
+lvcreate --type snapshot -s -l 20%FREE -n $lv3 $vg --virtualsize 256T
+lvcreate --type snapshot -s -l 20%FREE -n $lv4 $vg --virtualsize 256T
+lvcreate --type snapshot -s -l 20%FREE -n $lv5 $vg --virtualsize 256T
 
 aux extend_filter_LVMTEST
 
