@@ -18,6 +18,12 @@
 #include <signal.h>
 #include <sys/wait.h>
 
+struct device_id_list {
+	struct dm_list list;
+	struct device *dev;
+	char pvid[ID_LEN + 1];
+};
+
 const char *command_name(struct cmd_context *cmd)
 {
 	return cmd->command->name;
@@ -2074,11 +2080,37 @@ static int _get_arg_pvnames(struct cmd_context *cmd,
 	return ret_max;
 }
 
+static int _get_arg_devices(struct cmd_context *cmd,
+			    struct dm_list *arg_pvnames,
+			    struct dm_list *arg_devices)
+{
+	struct dm_str_list *sl;
+	struct device_id_list *dil;
+	int ret_max = ECMD_PROCESSED;
+
+	dm_list_iterate_items(sl, arg_pvnames) {
+		if (!(dil = dm_pool_alloc(cmd->mem, sizeof(*dil)))) {
+			log_error("device_id_list alloc failed.");
+			return ECMD_FAILED;
+		}
+
+		if (!(dil->dev = dev_cache_get(sl->str, cmd->filter))) {
+			log_error("Failed to find device for physical volume \"%s\".", sl->str);
+			ret_max = ECMD_FAILED;
+		} else {
+			strncpy(dil->pvid, dil->dev->pvid, ID_LEN);
+			dm_list_add(arg_devices, &dil->list);
+		}
+	}
+
+	return ret_max;
+}
+
 static int _get_all_devices(struct cmd_context *cmd, struct dm_list *all_devices)
 {
 	struct dev_iter *iter;
 	struct device *dev;
-	struct device_list *devl;
+	struct device_id_list *dil;
 	int r = ECMD_FAILED;
 
 	lvmcache_seed_infos_from_lvmetad(cmd);
@@ -2089,13 +2121,14 @@ static int _get_all_devices(struct cmd_context *cmd, struct dm_list *all_devices
 	}
 
 	while ((dev = dev_iter_get(iter))) {
-		if (!(devl = dm_pool_alloc(cmd->mem, sizeof(*devl)))) {
-			log_error("device_list alloc failed.");
+		if (!(dil = dm_pool_alloc(cmd->mem, sizeof(*dil)))) {
+			log_error("device_id_list alloc failed.");
 			goto out;
 		}
 
-		devl->dev = dev;
-		dm_list_add(all_devices, &devl->list);
+		strncpy(dil->pvid, dev->pvid, ID_LEN);
+		dil->dev = dev;
+		dm_list_add(all_devices, &dil->list);
 	}
 
 	r = ECMD_PROCESSED;
@@ -2104,13 +2137,13 @@ out:
 	return r;
 }
 
-static int _device_list_remove(struct dm_list *all_devices, struct device *dev)
+static int _device_list_remove(struct dm_list *devices, struct device *dev)
 {
-	struct device_list *devl;
+	struct device_id_list *dil;
 
-	dm_list_iterate_items(devl, all_devices) {
-		if (devl->dev == dev) {
-			dm_list_del(&devl->list);
+	dm_list_iterate_items(dil, devices) {
+		if (dil->dev == dev) {
+			dm_list_del(&dil->list);
 			return 1;
 		}
 	}
@@ -2118,12 +2151,36 @@ static int _device_list_remove(struct dm_list *all_devices, struct device *dev)
 	return 0;
 }
 
+static struct device_id_list *_device_list_find_dev(struct dm_list *devices, struct device *dev)
+{
+	struct device_id_list *dil;
+
+	dm_list_iterate_items(dil, devices) {
+		if (dil->dev == dev)
+			return dil;
+	}
+
+	return NULL;
+}
+
+static struct device_id_list *_device_list_find_pvid(struct dm_list *devices, struct physical_volume *pv)
+{
+	struct device_id_list *dil;
+
+	dm_list_iterate_items(dil, devices) {
+		if (id_equal((struct id *) dil->pvid, &pv->id))
+			return dil;
+	}
+
+	return NULL;
+}
+
 static int _process_device_list(struct cmd_context *cmd, struct dm_list *all_devices,
 				void *handle, process_single_pv_fn_t process_single_pv)
 {
 	struct physical_volume pv_dummy;
 	struct physical_volume *pv;
-	struct device_list *devl;
+	struct device_id_list *dil;
 	int ret_max = ECMD_PROCESSED;
 	int ret = 0;
 
@@ -2131,17 +2188,17 @@ static int _process_device_list(struct cmd_context *cmd, struct dm_list *all_dev
 	 * Pretend that each device is a PV with dummy values.
 	 * FIXME Formalise this extension or find an alternative.
 	 */
-	dm_list_iterate_items(devl, all_devices) {
+	dm_list_iterate_items(dil, all_devices) {
 		if (sigint_caught())
 			return_ECMD_FAILED;
 
 		memset(&pv_dummy, 0, sizeof(pv_dummy));
 		dm_list_init(&pv_dummy.tags);
 		dm_list_init(&pv_dummy.segments);
-		pv_dummy.dev = devl->dev;
+		pv_dummy.dev = dil->dev;
 		pv = &pv_dummy;
 
-		log_very_verbose("Processing device %s.", dev_name(devl->dev));
+		log_very_verbose("Processing device %s.", dev_name(dil->dev));
 
 		ret = process_single_pv(cmd, NULL, pv, handle);
 
@@ -2155,15 +2212,18 @@ static int _process_device_list(struct cmd_context *cmd, struct dm_list *all_dev
 static int _process_pvs_in_vg(struct cmd_context *cmd,
 			      struct volume_group *vg,
 			      struct dm_list *all_devices,
-			      struct dm_list *arg_pvnames,
+			      struct dm_list *arg_devices,
 			      struct dm_list *arg_tags,
-			      int process_all,
+			      int process_all_pvs,
+			      int process_all_devices,
 			      int skip,
 			      void *handle,
 			      process_single_pv_fn_t process_single_pv)
 {
 	struct physical_volume *pv;
 	struct pv_list *pvl;
+	struct device_id_list *dil;
+	struct device *dev_orig;
 	const char *pv_name;
 	int process_pv;
 	int dev_found;
@@ -2177,13 +2237,22 @@ static int _process_pvs_in_vg(struct cmd_context *cmd,
 		pv = pvl->pv;
 		pv_name = pv_dev_name(pv);
 
-		process_pv = process_all;
+		process_pv = process_all_pvs;
 
-		/* Remove each pvname as it is processed. */
-		if (!process_pv && !dm_list_empty(arg_pvnames) &&
-		    str_list_match_item(arg_pvnames, pv_name)) {
+		/* Remove each arg_devices entry as it is processed. */
+
+		if (!process_pv && !dm_list_empty(arg_devices) &&
+		    (dil = _device_list_find_dev(arg_devices, pv->dev))) {
 			process_pv = 1;
-			str_list_del(arg_pvnames, pv_name);
+			_device_list_remove(arg_devices, dil->dev);
+		}
+
+		/* Select the PV if the device arg has the same pvid. */
+
+		if (!process_pv && !dm_list_empty(arg_devices) &&
+		    (dil = _device_list_find_pvid(arg_devices, pv))) {
+			process_pv = 1;
+			_device_list_remove(arg_devices, dil->dev);
 		}
 
 		if (!process_pv && !dm_list_empty(arg_tags) &&
@@ -2219,13 +2288,71 @@ static int _process_pvs_in_vg(struct cmd_context *cmd,
 				if (ret > ret_max)
 					ret_max = ret;
 			}
+
+			/*
+			 * This is a very rare and obscure case where multiple
+			 * duplicate devices are specified on the command line
+			 * referring to this PV.  In this case we want to
+			 * process this PV once for each specified device.
+			 */
+
+			if (!skip && !dm_list_empty(arg_devices)) {
+				while ((dil = _device_list_find_pvid(arg_devices, pv))) {
+					_device_list_remove(arg_devices, dil->dev);
+
+					/*
+					 * Replace pv->dev with this dil->dev
+					 * in lvmcache so the duplicate dev
+					 * info will be reported.  FIXME: it
+					 * would be nicer to override pv->dev
+					 * without munging lvmcache content.
+					 */
+					dev_orig = pv->dev;
+					lvmcache_replace_dev(cmd, pv, dil->dev);
+
+					log_very_verbose("Processing PV %s device %s in VG %s.",
+							 pv_name, dev_name(dil->dev), vg->name);
+
+					ret = process_single_pv(cmd, vg, pv, handle);
+					if (ret != ECMD_PROCESSED)
+						stack;
+					if (ret > ret_max)
+						ret_max = ret;
+
+					/* Put the cache state back as it was. */
+					lvmcache_replace_dev(cmd, pv, dev_orig);
+				}
+			}
+
+			/*
+			 * This is another rare and obscure case where multiple
+			 * duplicate devices are being displayed by pvs -a, and
+			 * we want each of them to be displayed in the context
+			 * of this VG, so that this VG name appears next to it.
+			 */
+
+			if (process_all_devices && lvmcache_found_duplicate_pvs()) {
+				while ((dil = _device_list_find_pvid(all_devices, pv))) {
+					_device_list_remove(all_devices, dil->dev);
+
+					dev_orig = pv->dev;
+					lvmcache_replace_dev(cmd, pv, dil->dev);
+
+					ret = process_single_pv(cmd, vg, pv, handle);
+					if (ret != ECMD_PROCESSED)
+						stack;
+					if (ret > ret_max)
+						ret_max = ret;
+
+					lvmcache_replace_dev(cmd, pv, dev_orig);
+				}
+			}
 		}
 
 		/*
-		 * When processing only specific PV names, we can quit
-		 * once they've all been found.
+		 * When processing only specific PVs, we can quit once they've all been found.
 	 	 */
-		if (!process_all && dm_list_empty(arg_tags) && dm_list_empty(arg_pvnames))
+		if (!process_all_pvs && dm_list_empty(arg_tags) && dm_list_empty(arg_devices))
 			break;
 	}
 
@@ -2234,27 +2361,27 @@ static int _process_pvs_in_vg(struct cmd_context *cmd,
 
 /*
  * Iterate through all PVs in each listed VG.  Process a PV if
- * the name or tag matches arg_pvnames or arg_tags.  If both
- * arg_pvnames and arg_tags are empty, then process all PVs.
+ * its dev or tag matches arg_devices or arg_tags.  If both
+ * arg_devices and arg_tags are empty, then process all PVs.
  * No PV should be processed more than once.
  *
- * Each PV is removed from arg_pvnames and all_devices when it is
- * processed.  Any names remaining in arg_pvnames were not found, and
+ * Each PV is removed from arg_devices and all_devices when it is
+ * processed.  Any names remaining in arg_devices were not found, and
  * should produce an error.  Any devices remaining in all_devices were
- * not found and should be processed by process_all_devices().
+ * not found and should be processed by process_device_list().
  */
 static int _process_pvs_in_vgs(struct cmd_context *cmd, uint32_t flags,
 			       struct dm_list *all_vgnameids,
 			       struct dm_list *all_devices,
-			       struct dm_list *arg_pvnames,
+			       struct dm_list *arg_devices,
 			       struct dm_list *arg_tags,
-			       int process_all,
+			       int process_all_pvs,
+			       int process_all_devices,
 			       void *handle,
 			       process_single_pv_fn_t process_single_pv)
 {
 	struct volume_group *vg;
 	struct vgnameid_list *vgnl;
-	struct dm_str_list *sl;
 	const char *vg_name;
 	const char *vg_uuid;
 	int ret_max = ECMD_PROCESSED;
@@ -2282,8 +2409,9 @@ static int _process_pvs_in_vgs(struct cmd_context *cmd, uint32_t flags,
 		 * vg->pvs entries from devices list.
 		 */
 		
-		ret = _process_pvs_in_vg(cmd, vg, all_devices, arg_pvnames, arg_tags,
-					 process_all, skip, handle, process_single_pv);
+		ret = _process_pvs_in_vg(cmd, vg, all_devices, arg_devices, arg_tags,
+					 process_all_pvs, process_all_devices, skip,
+					 handle, process_single_pv);
 		if (ret != ECMD_PROCESSED)
 			stack;
 		if (ret > ret_max)
@@ -2295,16 +2423,10 @@ static int _process_pvs_in_vgs(struct cmd_context *cmd, uint32_t flags,
 			unlock_and_release_vg(cmd, vg, vg->name);
 
 		/* Quit early when possible. */
-		if (!process_all && dm_list_empty(arg_tags) && dm_list_empty(arg_pvnames))
+		if (!process_all_pvs && dm_list_empty(arg_tags) && dm_list_empty(arg_devices))
 			return ret_max;
 	}
 
-	/* Return an error if a pvname arg was not found. */
-	dm_list_iterate_items(sl, arg_pvnames) {
-		log_error("Failed to find physical volume \"%s\".", sl->str);
-		ret_max = ECMD_FAILED;
-	}
-
 	return ret_max;
 }
 
@@ -2317,8 +2439,10 @@ int process_each_pv(struct cmd_context *cmd,
 {
 	struct dm_list arg_tags;	/* str_list */
 	struct dm_list arg_pvnames;	/* str_list */
+	struct dm_list arg_devices;	/* device_id_list */
 	struct dm_list all_vgnameids;	/* vgnameid_list */
-	struct dm_list all_devices;	/* device_list */
+	struct dm_list all_devices;	/* device_id_list */
+	struct device_id_list *dil;
 	int process_all_pvs;
 	int process_all_devices;
 	int ret_max = ECMD_PROCESSED;
@@ -2326,16 +2450,31 @@ int process_each_pv(struct cmd_context *cmd,
 
 	dm_list_init(&arg_tags);
 	dm_list_init(&arg_pvnames);
+	dm_list_init(&arg_devices);
 	dm_list_init(&all_vgnameids);
 	dm_list_init(&all_devices);
 
+	/*
+	 * Read all the vgs first because this has the effect of initializing
+	 * other device/lvmcache info that is needed when creating device lists.
+	 */
+	if ((ret = _get_vgnameids_on_system(cmd, &all_vgnameids, only_this_vgname, 1) != ECMD_PROCESSED)) {
+		stack;
+		return ret;
+	}
+
 	/*
 	 * Create two lists from argv:
 	 * arg_pvnames: pvs explicitly named in argv
 	 * arg_tags: tags explicitly named in argv
+	 *
+	 * Then convert arg_pvnames, which are free-form, user-specified,
+	 * names/paths into arg_devices which can be used to match below.
 	 */
-	if ((ret = _get_arg_pvnames(cmd, argc, argv, &arg_pvnames, &arg_tags)) != ECMD_PROCESSED)
+	if ((ret = _get_arg_pvnames(cmd, argc, argv, &arg_pvnames, &arg_tags)) != ECMD_PROCESSED) {
+		stack;
 		return ret;
+	}
 
 	process_all_pvs = dm_list_empty(&arg_pvnames) && dm_list_empty(&arg_tags);
 
@@ -2343,6 +2482,11 @@ int process_each_pv(struct cmd_context *cmd,
 			   (cmd->command->flags & ENABLE_ALL_DEVS) &&
 			   arg_count(cmd, all_ARG);
 
+	if ((ret = _get_arg_devices(cmd, &arg_pvnames, &arg_devices) != ECMD_PROCESSED)) {
+		/* get_arg_devices reports the error for any PV names not found. */
+		ret_max = ECMD_FAILED;
+	}
+
 	/*
 	 * If the caller wants to process all devices (not just PVs), then all PVs
 	 * from all VGs are processed first, removing them from all_devices.  Then
@@ -2353,19 +2497,20 @@ int process_each_pv(struct cmd_context *cmd,
 		return ret;
 	}
 
-	if ((ret = _get_vgnameids_on_system(cmd, &all_vgnameids, only_this_vgname, 1) != ECMD_PROCESSED)) {
-		stack;
-		return ret;
-	}
-
 	ret = _process_pvs_in_vgs(cmd, flags, &all_vgnameids, &all_devices,
-				  &arg_pvnames, &arg_tags, process_all_pvs,
+				  &arg_devices, &arg_tags,
+				  process_all_pvs, process_all_devices,
 				  handle, process_single_pv);
 	if (ret != ECMD_PROCESSED)
 		stack;
 	if (ret > ret_max)
 		ret_max = ret;
 
+	dm_list_iterate_items(dil, &arg_devices) {
+		log_error("Failed to find physical volume \"%s\".", dev_name(dil->dev));
+		ret_max = ECMD_FAILED;
+	}
+
 	if (!process_all_devices)
 		goto out;
 
