@@ -23,27 +23,26 @@ static int _monitor_lvs_in_vg(struct cmd_context *cmd,
 {
 	struct lv_list *lvl;
 	struct logical_volume *lv;
-	struct lvinfo info;
 	int r = 1;
 
 	dm_list_iterate_items(lvl, &vg->lvs) {
 		lv = lvl->lv;
 
 		if (!lv_info(cmd, lv, lv_is_thin_pool(lv) ? 1 : 0,
-			     &info, 0, 0) ||
-		    !info.exists)
+			     NULL, 0, 0))
 			continue;
 		/*
 		 * FIXME: Need to consider all cases... PVMOVE, etc
 		 */
-		if (lv->status & PVMOVE)
+		if (lv_is_pvmove(lv))
 			continue;
 
 		if (!monitor_dev_for_events(cmd, lv, 0, reg)) {
 			r = 0;
 			continue;
-		} else
-			(*count)++;
+		}
+
+		(*count)++;
 	}
 
 	return r;
@@ -67,7 +66,7 @@ static int _poll_lvs_in_vg(struct cmd_context *cmd,
 			lv_active = info.exists;
 
 		if (lv_active &&
-		    (lv->status & (PVMOVE|CONVERTING|MERGING))) {
+		    (lv_is_pvmove(lv) || lv_is_converting(lv) || lv_is_merging(lv))) {
 			lv_spawn_background_polling(cmd, lv);
 			count++;
 		}
@@ -98,10 +97,6 @@ static int _activate_lvs_in_vg(struct cmd_context *cmd, struct volume_group *vg,
 		if (!lv_is_visible(lv))
 			continue;
 
-		/* Cache pool cannot be activated */
-		if (lv_is_cache_pool(lv))
-			continue;
-
 		/* If LV is sparse, activate origin instead */
 		if (lv_is_cow(lv) && lv_is_virtual_origin(origin_from_cow(lv)))
 			lv = origin_from_cow(lv);
@@ -121,7 +116,7 @@ static int _activate_lvs_in_vg(struct cmd_context *cmd, struct volume_group *vg,
 
 		/* Can't deactivate a pvmove LV */
 		/* FIXME There needs to be a controlled way of doing this */
-		if ((lv->status & PVMOVE) && !is_change_activating(activate))
+		if (lv_is_pvmove(lv) && !is_change_activating(activate))
 			continue;
 
 		if (lv_activation_skip(lv, activate, arg_count(cmd, ignoreactivationskip_ARG)))
@@ -152,6 +147,7 @@ static int _activate_lvs_in_vg(struct cmd_context *cmd, struct volume_group *vg,
 	}
 
 	sigint_restore();
+	sync_local_dev_names(vg->cmd);  /* Wait until devices are available */
 
 	if (expected_count)
 		log_verbose("%s %d logical volumes in volume group %s",
@@ -198,7 +194,6 @@ int vgchange_activate(struct cmd_context *cmd, struct volume_group *vg,
 {
 	int lv_open, active, monitored = 0, r = 1;
 	const struct lv_list *lvl;
-	struct lvinfo info;
 	int do_activate = is_change_activating(activate);
 
 	/*
@@ -211,8 +206,7 @@ int vgchange_activate(struct cmd_context *cmd, struct volume_group *vg,
 	if (!do_activate && (lv_open = lvs_in_vg_opened(vg))) {
 		dm_list_iterate_items(lvl, &vg->lvs)
 			if (lv_is_visible(lvl->lv) &&
-			    lv_info(cmd, lvl->lv, 0, &info, 1, 0) &&
-			    !lv_check_not_in_use(cmd, lvl->lv, &info)) {
+			    !lv_check_not_in_use(lvl->lv)) {
 				log_error("Can't deactivate volume group \"%s\" with %d open "
 					  "logical volume(s)", vg->name, lv_open);
 				return 0;
@@ -236,8 +230,10 @@ int vgchange_activate(struct cmd_context *cmd, struct volume_group *vg,
 		}
 	}
 
-	if (!_activate_lvs_in_vg(cmd, vg, activate))
+	if (!_activate_lvs_in_vg(cmd, vg, activate)) {
+		stack;
 		r = 0;
+	}
 
 	/* Print message only if there was not found a missing VG */
 	if (!vg->cmd_missing_vgs)
@@ -278,7 +274,7 @@ static int _vgchange_alloc(struct cmd_context *cmd, struct volume_group *vg)
 static int _vgchange_resizeable(struct cmd_context *cmd,
 				struct volume_group *vg)
 {
-	int resizeable = !strcmp(arg_str_value(cmd, resizeable_ARG, "n"), "y");
+	int resizeable = arg_int_value(cmd, resizeable_ARG, 0);
 
 	if (resizeable && vg_is_resizeable(vg)) {
 		log_error("Volume group \"%s\" is already resizeable",
@@ -303,7 +299,7 @@ static int _vgchange_resizeable(struct cmd_context *cmd,
 static int _vgchange_clustered(struct cmd_context *cmd,
 			       struct volume_group *vg)
 {
-	int clustered = !strcmp(arg_str_value(cmd, clustered_ARG, "n"), "y");
+	int clustered = arg_int_value(cmd, clustered_ARG, 0);
 
 	if (clustered && (vg_is_clustered(vg))) {
 		log_error("Volume group \"%s\" is already clustered",
@@ -479,7 +475,9 @@ static int vgchange_single(struct cmd_context *cmd, const char *vg_name,
 			   struct volume_group *vg,
 			   void *handle __attribute__((unused)))
 {
+	int ret = ECMD_PROCESSED;
 	unsigned i;
+	struct lv_list *lvl;
 
 	static const struct {
 		int arg;
@@ -534,6 +532,31 @@ static int vgchange_single(struct cmd_context *cmd, const char *vg_name,
 		backup(vg);
 
 		log_print_unless_silent("Volume group \"%s\" successfully changed", vg->name);
+
+		/* FIXME: fix clvmd bug and take DLM lock for non clustered VGs. */
+		if (arg_is_set(cmd, clustered_ARG) &&
+		    vg_is_clustered(vg) && /* just switched to clustered */
+		    locking_is_clustered() &&
+		    locking_supports_remote_queries())
+			dm_list_iterate_items(lvl, &vg->lvs) {
+				if ((lv_lock_holder(lvl->lv) != lvl->lv) ||
+				    !lv_is_active(lvl->lv))
+					continue;
+
+				if (!activate_lv_excl_local(cmd, lvl->lv) ||
+				    !lv_is_active_exclusive_locally(lvl->lv)) {
+					log_error("Can't reactive logical volume %s, "
+						  "please fix manually.",
+						  display_lvname(lvl->lv));
+					ret = ECMD_FAILED;
+				}
+
+				if (lv_is_mirror(lvl->lv))
+					/* Give hint for clustered mirroring */
+					log_print_unless_silent("For clustered mirroring of %s "
+								"deactivation and activation is needed.",
+								display_lvname(lvl->lv));
+			}
 	}
 
 	if (arg_count(cmd, activate_ARG)) {
@@ -561,7 +584,7 @@ static int vgchange_single(struct cmd_context *cmd, const char *vg_name,
 		if (!_vgchange_background_polling(cmd, vg))
 			return_ECMD_FAILED;
 
-        return ECMD_PROCESSED;
+        return ret;
 }
 
 int vgchange(struct cmd_context *cmd, int argc, char **argv)
@@ -662,7 +685,7 @@ int vgchange(struct cmd_context *cmd, int argc, char **argv)
 			 * direct activation instead of autoactivation.
 			 */
 			log_warn("lvmetad is not active yet, using direct activation during sysinit");
-			lvmetad_set_active(0);
+			lvmetad_set_active(cmd, 0);
 		} else if (lvmetad_active()) {
 			/*
 			 * If lvmetad is active already, we want
